@@ -3,7 +3,7 @@ __all__ = ['ResultException', 'ResultType', 'Result', 'Ok', 'Err', 'wrap_result'
 import asyncio
 import types
 from typing import Any, Callable
-
+from inspect import iscoroutinefunction
 
 class ResultException(BaseException):
     pass
@@ -200,12 +200,22 @@ def wrap_result(res: Result):
     """
     def outer(f):
         def inner(*args, **kwargs):
-            try:
-                v = f(*args, **kwargs)
-                return Ok(v)
-            except BaseException as e: #  TODO check it was Exception not BaseException
-                return Err(e)
+            if iscoroutinefunction(f):
+                async def a():
+                    try:
+                        v = await f(*args, **kwargs)
+                        return Ok(v)
+                    except BaseException as e: #  TODO check it was Exception not BaseException
+                        return Err(e)  
+                return a()
+            else:
+                try:
+                    v = f(*args, **kwargs)
+                    return Ok(v)
+                except BaseException as e: #  TODO check it was Exception not BaseException
+                    return Err(e)
 
         return inner
 
     return outer
+
